body,html{height:100%}body{min-width:800px;background-color:#f3f3f3;font-family:monospace;font-size:small}body,p{margin:0}ul{padding:0}li{display:inline-block;list-style:none;padding-left:10px}select{border-radius:3px;padding:1px 3px}.frame{background-color:#fff;margin:6px;padding:6px;border-radius:5px}.frame>div.left>img{vertical-align:middle}.left{float:left}.right{float:right}.clear-fix{*overflow:hidden;*zoom:1}.clear-fix:after{display:table;content:"";width:0;clear:both}button.skip{opacity:0;width:20px;height:20px;border:0}button.skip:hover{opacity:1}.skipped{height:6px;padding:0}.skipped>.left,.skipped>.right{display:none}.wrapper{height:100%;flex-grow:1;display:flex;flex-flow:row}.left-panel{flex-grow:1;margin-right:10px}.right-panel{flex-basis:300px;padding:10px}.v-slider{position:relative;vertical-align:middle}.v-slider-bar{background-color:#333;position:absolute}.v-slider-point{background-color:#bbb;position:absolute}.highlight{background:none repeat scroll 0 0 #ffffe0;border:1px solid #e6db55}#ui-overlap{position:relative}#ui-overlap>div,.uibox{position:absolute}.uibox{border:1px solid red}@keyframes breath{0%{opacity:1;box-shadow:0 0 10px 10px #dc143c}50%{opacity:.4;box-shadow:0 0 1px 1px #dc143c}to{opacity:1;box-shadow:0 0 10px 10px #dc143c}}.point{animation:breath 2s ease-in-out infinite;background-color:#fff;border:1px solid red;position:absolute}.chop{display:inline-block;vertical-align:middle}.rect{position:absolute;border:1px solid red}.full{width:100%;height:100%}.resizer{position:absolute;right:0;bottom:0;height:5px;width:5px;border:1px solid red}.resizer:hover{cursor:se-resize}.nav{margin-left:12px}.nav>li{float:left;margin-right:8px;line-height:22px}.toolbar{flex-basis:60px}.tool{margin:5px}.tool img{width:30px;height:30px}.selected{background:none repeat scroll 0 0 #e00}#app{color:#2c3e50;font-family:Source Sans Pro,Helvetica,sans-serif}#app a{color:#42b983;text-decoration:none}.scroll{height:100%;overflow-y:auto}.scroll::-webkit-scrollbar{width:16px}.scroll-wrapper{height:100%;position:relative}.scroll-bottom-more,.scroll-top-more{position:absolute;height:10px;left:5px;right:5px;margin:0 auto;text-align:center}.scroll-top-more{top:-10px}.scroll-bottom-more{bottom:-10px}.arrow-up{border-bottom:10px solid blue}.arrow-down,.arrow-up{margin:0 auto;width:0;height:0;border-left:10px solid transparent;border-right:10px solid transparent}.arrow-down{border-top:10px solid blue}